<?php

namespace Crm\Funil\Policies;

use Litepie\User\Contracts\UserPolicy;
use Crm\Funil\Models\Funil;

class FunilPolicy
{

    /**
     * Determine if the given user can view the funil.
     *
     * @param UserPolicy $user
     * @param Funil $funil
     *
     * @return bool
     */
    public function view(UserPolicy $user, Funil $funil)
    {
        if ($user->canDo('funil.funil.view') && $user->isAdmin()) {
            return true;
        }

        return $funil->user_id == user_id() && $funil->user_type == user_type();
    }

    /**
     * Determine if the given user can create a funil.
     *
     * @param UserPolicy $user
     * @param Funil $funil
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('funil.funil.create');
    }

    /**
     * Determine if the given user can update the given funil.
     *
     * @param UserPolicy $user
     * @param Funil $funil
     *
     * @return bool
     */
    public function update(UserPolicy $user, Funil $funil)
    {
        if ($user->canDo('funil.funil.edit') && $user->isAdmin()) {
            return true;
        }

        return $funil->user_id == user_id() && $funil->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given funil.
     *
     * @param UserPolicy $user
     * @param Funil $funil
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Funil $funil)
    {
        return $funil->user_id == user_id() && $funil->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given funil.
     *
     * @param UserPolicy $user
     * @param Funil $funil
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Funil $funil)
    {
        if ($user->canDo('funil.funil.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given funil.
     *
     * @param UserPolicy $user
     * @param Funil $funil
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Funil $funil)
    {
        if ($user->canDo('funil.funil.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
